/*
 * Use at your own risk.
 *
 * Copyright (C) 2005-2008 Sourcefire, Inc.
 * 
 * This file is autogenerated via rules2c, by Brian Caswell <bmc@sourcefire.com>
 */

#include "sf_snort_plugin_api.h"
#include "sf_snort_packet.h"

/* declare detection functions */
int rule99999eval(void *p);

/* declare rule data structures */
/* precompile the stuff that needs pre-compiled */
/* byte_test:size 1, value 128, operator &, offset 2; */
static ByteData rule99999byte_test0 = 
{
    1, /* size */
    CHECK_AND, /* operator */
    128, /* value */
    2, /* offset */
    0, /*multiplier */
    BYTE_BIG_ENDIAN|CONTENT_BUF_NORMALIZED|EXTRACT_AS_BYTE /* flags */
};

static RuleOption rule99999option0 = 
{
    OPTION_TYPE_BYTE_TEST,
    {
        &rule99999byte_test0
    }
};
/* byte_test:size 2, value 0, operator >, offset 6; */
static ByteData rule99999byte_test1 = 
{
    2, /* size */
    CHECK_GT, /* operator */
    0, /* value */
    6, /* offset */
    0, /*multiplier */
    BYTE_BIG_ENDIAN|CONTENT_BUF_NORMALIZED|EXTRACT_AS_BYTE /* flags */
};

static RuleOption rule99999option1 = 
{
    OPTION_TYPE_BYTE_TEST,
    {
        &rule99999byte_test1
    }
};

/* references for sid 99999 */
/* reference: bugtraq "31881"; */
static RuleReference rule99999ref1 = 
{
    "bugtraq", /* type */
    "31881" /* value */
};

/* reference: cve "2008-2469"; */
static RuleReference rule99999ref2 = 
{
    "cve", /* type */
    "2008-2469" /* value */
};

static RuleReference *rule99999refs[] =
{
    &rule99999ref1,
    &rule99999ref2,
    NULL
};
#ifdef HAS_METADATA
/* metadata for sid 99999 */
/* metadata:; */


static RuleMetaData *rule99999metadata[] =
{
    NULL
};
#endif
RuleOption *rule99999options[] =
{
    &rule99999option0,
    &rule99999option1,
    NULL
};

Rule rule99999 = {
   
   /* rule header, akin to => tcp any any -> any any               */{
       IPPROTO_UDP, /* proto */
       "$EXTERNAL_NET", /* SRCIP     */
       "53", /* SRCPORT   */
       1, /* DIRECTION */
       "$HOME_NET", /* DSTIP     */
       "any", /* DSTPORT   */
   },
   /* metadata */
   { 
       3,  /* genid (HARDCODED!!!) */
       99999, /* sigid */
       1, /* revision */
   
       "attempted-admin", /* classification */
       0,  /* hardcoded priority XXX NOT PROVIDED BY GRAMMAR YET! */
       "DNS invalid txt record character string size",     /* message */
       rule99999refs /* ptr to references */
   #ifdef HAS_METADATA
       ,rule99999metadata
   #endif
   },
   rule99999options, /* ptr to rule options */
   &rule99999eval, /* use the built in detection function */
   0 /* am I initialized yet? */
};


/* detection functions */
int rule99999eval(void *p) {
    const u_int8_t *cursor_raw = 0;
    const u_int8_t *end_of_payload;
    u_int16_t num_queries, num_answers;
    u_int16_t i, skip;
    u_int16_t data_size;
    u_int8_t txt_size;
    SFSnortPacket *sp = (SFSnortPacket *) p;

    if(sp == NULL)
        return RULE_NOMATCH;

    if(sp->payload == NULL)
        return RULE_NOMATCH;
    
    // Ensure that we have at least the 12 bytes of header before parsing it
    if (sp->payload_size < 12) {
        return RULE_NOMATCH;
    }

    // byte_test:size 1, value 128, operator &, offset 2;
    if (byteTest(p, rule99999options[0]->option_u.byte, cursor_raw) <= 0) {
        return RULE_NOMATCH;
    }
    // byte_test:size 2, value 0, operator >, offset 6;
    if (byteTest(p, rule99999options[1]->option_u.byte, cursor_raw) <= 0) {
        return RULE_NOMATCH;
    }

    // Start by looking at the number of queries in the packet
    cursor_raw = sp->payload + 4;

    // Extract number of queries, so we can skip them later
    num_queries = *cursor_raw++ << 8;
    num_queries |= *cursor_raw++;

    // Now extract number of answers, so we can check them later
    num_answers = *cursor_raw++ << 8;
    num_answers |= *cursor_raw++;

    // Skip to the start of the queries
    cursor_raw += 4;

    // Useful for performing packet size checks from here on out
    end_of_payload = sp->payload + sp->payload_size;

    // Loop over each query in the packet, looking for the null byte
    // that terminates the name; skip it plus the following four bytes of type/class
    for (i = 0; i < num_queries; i++) {
        while ((cursor_raw < end_of_payload) && (*cursor_raw != 0)) {
            cursor_raw++; 
        }
        if ((cursor_raw + 5) >= end_of_payload) {
            return RULE_NOMATCH;
        }
        cursor_raw += 5;
    }

    // Now we process the answers, looking for TXT records, and skipping any other types
    for (i = 0; i < num_answers; i++) {
        // Names should always be compressed, so we'll skip two bytes for the name
        if ((cursor_raw + 2) >= end_of_payload) {
            return RULE_NOMATCH;
        }
        cursor_raw += 2;
        // Reset the flag for skipping non-TXT records
        if ((cursor_raw + 2) >= end_of_payload) {
            return RULE_NOMATCH;
        }
        skip = 0;
        if ((*cursor_raw != 0) || (*cursor_raw++ != 0x10)) {
            skip = 1;
        }
        // Skip class & TTL
        if ((cursor_raw + 6) >= end_of_payload) {
            return RULE_NOMATCH;
        }
        cursor_raw += 6;

        // Get the size of the record data
        if ((cursor_raw + 2) >= end_of_payload) {
            return RULE_NOMATCH;
        }
        data_size = *cursor_raw++ << 8;
        data_size |= *cursor_raw++;
        if ((cursor_raw + data_size) >= end_of_payload) {
            return RULE_NOMATCH;
        }
        if (skip == 1) {
            cursor_raw += data_size;
        }
        // Get the txt data size
        txt_size = *cursor_raw;
        // If txt_size is not = data_size -1, alert
        if ((txt_size - 1) != data_size) {
            return RULE_MATCH;
        }
        // Otherwise, skip over the record
        cursor_raw += data_size;
    }
    
    return RULE_NOMATCH;
}

/*
Rule *rules[] = {
    &rule99999,
    NULL
};
*/
